@@ -23,7 +23,7 @@
 
 #include "config.h"
 
-#include "run-auto-test.h"
+#include "test-suite.h"
 
 #include "egg/egg-asn1.h"
 #include "egg/egg-oid.h"
@@ -55,7 +55,7 @@ DEFINE_SETUP(asn1_tree)
 
 	/* -------- */
 	
-	data_cert = test_data_read ("test-certificate-1.der", &n_data_cert);
+	data_cert = testing_data_read ("test-certificate-1.der", &n_data_cert);
 
 	/* We'll be catching this error later */
 	pkix = egg_asn1_get_pkix_asn1type ();
@@ -380,103 +380,3 @@ DEFINE_TEST(read_date)
 	g_assert_cmpint (date.month, ==, 12);
 	g_assert_cmpint (date.year, ==, 2020);
 }
-
-DEFINE_TEST(read_dn)
-{
-	gchar *dn;
-	
-	dn = egg_asn1_read_dn (asn1_cert, "tbsCertificate.issuer.rdnSequence");
-	g_assert (dn != NULL);
-	g_assert_cmpstr (dn, ==, "C=ZA, ST=Western Cape, L=Cape Town, O=Thawte Consulting, OU=Certification Services Division, CN=Thawte Personal Premium CA, EMAIL=personal-premium@thawte.com");
-	
-	g_free (dn);
-	
-	dn = egg_asn1_read_dn (asn1_cert, "tbsCertificate.nonExistant");
-	g_assert (dn == NULL);
-}
-
-DEFINE_TEST(dn_value)
-{
-	const guchar value[] = { 0x13, 0x1a, 0x54, 0x68, 0x61, 0x77, 0x74, 0x65, 0x20, 0x50, 0x65, 0x72, 0x73, 0x6f, 0x6e, 0x61, 0x6c, 0x20, 0x50, 0x72, 0x65, 0x6d, 0x69, 0x75, 0x6d, 0x20, 0x43, 0x41 };
-	gsize n_value = 28;
-	GQuark oid;
-	gchar *text;
-	
-	/* Some printable strings */
-	oid = g_quark_from_static_string ("2.5.4.3");
-	text = egg_asn1_dn_print_value (oid, value, n_value);
-	g_assert_cmpstr (text, ==, "Thawte Personal Premium CA");
-	g_free (text);
-
-	/* Unknown oid */
-	oid = g_quark_from_static_string ("1.1.1.1.1.1");
-	text = egg_asn1_dn_print_value (oid, value, n_value);
-	g_assert_cmpstr (text, ==, "#131A54686177746520506572736F6E616C205072656D69756D204341");
-	g_free (text);
-}
-
-static int last_index = 0;
-
-static void
-concatenate_dn (guint index, GQuark oid, const guchar *value, gsize n_value, gpointer user_data)
-{
-	GString *dn = user_data;
-	gchar *text;
-	
-	g_assert (oid);
-	g_assert (value);
-	g_assert (n_value);
-	
-	g_assert (index == last_index);
-	++last_index;
-	
-	if (index != 1) {
-		g_string_append (dn, ", ");
-	}
-	
-	g_string_append (dn, egg_oid_get_name (oid));
-	g_string_append_c (dn, '=');
-	
-	text = egg_asn1_dn_print_value (oid, value, n_value);
-	g_string_append (dn, text);
-	g_free (text);
-}
-
-DEFINE_TEST(parse_dn)
-{
-	GString *dn = g_string_new ("");
-	last_index = 1;
-	
-	if (!egg_asn1_dn_parse (asn1_cert, "tbsCertificate.issuer.rdnSequence", concatenate_dn, dn))
-		g_assert_not_reached ();
-	
-	g_assert_cmpstr (dn->str, ==, "C=ZA, ST=Western Cape, L=Cape Town, O=Thawte Consulting, OU=Certification Services Division, CN=Thawte Personal Premium CA, EMAIL=personal-premium@thawte.com");
-	g_string_free (dn, TRUE);
-}
-
-DEFINE_TEST(read_dn_part)
-{
-	gchar *value;
-	
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.issuer.rdnSequence", "CN");
-	g_assert (value != NULL);
-	g_assert_cmpstr (value, ==, "Thawte Personal Premium CA");
-	g_free (value);
-
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.issuer.rdnSequence", "2.5.4.8");
-	g_assert (value != NULL);
-	g_assert_cmpstr (value, ==, "Western Cape");
-	g_free (value);
-	
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.nonExistant", "CN");
-	g_assert (value == NULL);
-
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.issuer.rdnSequence", "DC");
-	g_assert (value == NULL);
-
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.issuer.rdnSequence", "0.0.0.0");
-	g_assert (value == NULL);
-
-	value = egg_asn1_read_dn_part (asn1_cert, "tbsCertificate.issuer.rdnSequence", "2.5.4.9");
-	g_assert (value == NULL);
-}
