@@ -59,8 +59,8 @@ attribute_init (GP11Attribute *attr, gulong attr_type,
 	memset (attr, 0, sizeof (GP11Attribute));
 	attr->type = attr_type;
 	attr->length = length;
-	if (value && length) {
-		attr->value = (allocator) (NULL, length);
+	if (value) {
+		attr->value = (allocator) (NULL, length ? length : 1);
 		g_assert (attr->value);
 		memcpy (attr->value, value, length);
 	}
@@ -653,34 +653,39 @@ struct _GP11Attributes {
 
 /**
  * GP11_BOOLEAN:
+ *
  * The attribute data is a gboolean. Used with variable argument functions.
- */
+ **/
 
 /**
  * GP11_ULONG:
+ *
  * The attribute data is a gulong. Used with variable argument functions.  
- */
+ **/
 
 /**
  * GP11_STRING:
+ *
  * The attribute data is a gchar. Used with variable argument functions.  
- */
+ **/
 
 /**
  * GP11_DATE:
+ *
  * The attribute data is a GDate. Used with variable argument functions.  
- */
+ **/
 
 /**
  * GP11_DATE:
+ *
  * Signifies that no more attributes follow. Used with variable argument functions.  
- */
+ **/
 
 /**
  * GP11_ATTRIBUTES_TYPE:
  * 
  * A boxed type that can be used to hold a GP11Attributes object.
- */
+ **/
 
 /**
  * GP11Allocator:
@@ -691,8 +696,10 @@ struct _GP11Attributes {
  * 
  * This is a function that acts like g_realloc. Specifically it frees when length is 
  * set to zero, it allocates when data is set to NULL, and it reallocates when both 
- * are valid.   
- */
+ * are valid.
+ *
+ * Returns: The allocated memory, or NULL when freeing.
+ **/
 
 /**
  * gp11_attributes_get_boxed_type:
@@ -755,9 +762,10 @@ gp11_attributes_new_full (GP11Allocator allocator)
 /**
  * gp11_attributes_new_empty:
  * @attr_type: The first attribute type to add as empty.
+ * @...: The arguments should be values of attribute types, terminated with GP11_INVALID.
  * 
  * Creates an GP11Attributes array with empty attributes. The arguments 
- * should be values of attribute types, terminated with -1.
+ * should be values of attribute types, terminated with GP11_INVALID.
  * 
  * Return value: The new attributes array. When done with the array 
  * release it with gp11_attributes_unref().
@@ -834,11 +842,11 @@ initialize_from_valist (GP11Allocator allocator, gulong type, va_list va)
 
 /**
  * gp11_attributes_newv:
- * 
+ * @attr_type: The first attribute type.
+ * @...: The arguments must be triples of: attribute type, data type, value
+ *
  * Create a new GP11Attributes array, containing a list of attributes.
  * 
- * The arguments must be triples of: attribute type, data type, value
- * 
  * <para>The variable argument list should contain:
  * 	<variablelist>
  *		<varlistentry>
@@ -862,20 +870,20 @@ initialize_from_valist (GP11Allocator allocator, gulong type, va_list va)
  * release it with gp11_attributes_unref().
  **/
 GP11Attributes*
-gp11_attributes_newv (gulong first_type, ...)
+gp11_attributes_newv (gulong attr_type, ...)
 {
 	GP11Attributes *attrs;
 	va_list va;
 	
-	va_start (va, first_type);
-	attrs = initialize_from_valist (g_realloc, first_type, va);
+	va_start (va, attr_type);
+	attrs = initialize_from_valist (g_realloc, attr_type, va);
 	va_end (va);
 	
 	return attrs;
 }
 
 /**
- * gp11_attributes_newv:
+ * gp11_attributes_new_valist:
  * @allocator: Memory allocator for attribute data, or NULL for default.
  * @va: Variable argument containing attributes to add. 
  * 
@@ -955,16 +963,19 @@ attributes_push (GP11Attributes *attrs)
  * Add the specified attribute to the array. 
  * 
  * The value stored in the attribute will be copied.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add (GP11Attributes *attrs, GP11Attribute *attr)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs && attrs->array);
-	g_return_if_fail (!attrs->locked);
-	g_return_if_fail (attr);
+	g_return_val_if_fail (attrs && attrs->array, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
+	g_return_val_if_fail (attr, NULL);
 	added = attributes_push (attrs);
 	attribute_init_copy (added, attr, attrs->allocator);
+	return added;
 }
 
 /**
@@ -977,16 +988,19 @@ gp11_attributes_add (GP11Attributes *attrs, GP11Attribute *attr)
  * Add an attribute with the specified type and value to the array. 
  * 
  * The value stored in the attribute will be copied.
+ *
+ * Return value: The attribute that was added.
  **/
-void 
+GP11Attribute*
 gp11_attributes_add_data (GP11Attributes *attrs, gulong attr_type,
                           gconstpointer value, gsize length)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	attribute_init (added, attr_type, value, length, attrs->allocator);
+	return added;
 }
 
 /**
@@ -995,15 +1009,18 @@ gp11_attributes_add_data (GP11Attributes *attrs, gulong attr_type,
  * @attr_type: The type of attribute to add.
  * 
  * Add an attribute with the specified type and an 'invalid' value to the array. 
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_invalid (GP11Attributes *attrs, gulong attr_type)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	gp11_attribute_init_invalid (added, attr_type);	
+	return added;
 }
 
 /**
@@ -1012,15 +1029,18 @@ gp11_attributes_add_invalid (GP11Attributes *attrs, gulong attr_type)
  * @attr_type: The type of attribute to add.
  * 
  * Add an attribute with the specified type, with empty data.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_empty (GP11Attributes *attrs, gulong attr_type)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	gp11_attribute_init_empty (added, attr_type);		
+	return added;
 }
 
 /**
@@ -1032,15 +1052,18 @@ gp11_attributes_add_empty (GP11Attributes *attrs, gulong attr_type)
  * Add an attribute with the specified type and value to the array. 
  * 
  * The value will be stored as a CK_BBOOL PKCS#11 style attribute.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_boolean (GP11Attributes *attrs, gulong attr_type, gboolean value)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	attribute_init_boolean (added, attr_type, value, attrs->allocator);
+	return added;
 }
 
 /**
@@ -1052,15 +1075,18 @@ gp11_attributes_add_boolean (GP11Attributes *attrs, gulong attr_type, gboolean v
  * Add an attribute with the specified type and value to the array. 
  * 
  * The value will be copied into the attribute.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_string (GP11Attributes *attrs, gulong attr_type, const gchar *value)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	attribute_init_string (added, attr_type, value, attrs->allocator);
+	return added;
 }
 
 /**
@@ -1072,15 +1098,18 @@ gp11_attributes_add_string (GP11Attributes *attrs, gulong attr_type, const gchar
  * Add an attribute with the specified type and value to the array. 
  * 
  * The value will be stored as a CK_DATE PKCS#11 style attribute.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_date (GP11Attributes *attrs, gulong attr_type, const GDate *value)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	attribute_init_date (added, attr_type, value, attrs->allocator);
+	return added;
 }
 
 /**
@@ -1092,15 +1121,18 @@ gp11_attributes_add_date (GP11Attributes *attrs, gulong attr_type, const GDate *
  * Add an attribute with the specified type and value to the array. 
  * 
  * The value will be stored as a CK_ULONG PKCS#11 style attribute.
+ *
+ * Return value: The attribute that was added.
  **/
-void
+GP11Attribute*
 gp11_attributes_add_ulong (GP11Attributes *attrs, gulong attr_type, gulong value)
 {
 	GP11Attribute *added;
-	g_return_if_fail (attrs);
-	g_return_if_fail (!attrs->locked);
+	g_return_val_if_fail (attrs, NULL);
+	g_return_val_if_fail (!attrs->locked, NULL);
 	added = attributes_push (attrs);
 	attribute_init_ulong (added, attr_type, value, attrs->allocator);
+	return added;
 }
 
 /**
@@ -1265,9 +1297,11 @@ gp11_attributes_find_date (GP11Attributes *attrs, gulong attr_type, GDate *value
 /**
  * gp11_attributes_ref:
  * @attrs: An attribute array
- * 
+ *
  * Reference this attributes array.
- */
+ *
+ * Returns: The attributes.
+ **/
 GP11Attributes*
 gp11_attributes_ref (GP11Attributes *attrs)
 {
